import db from '../db.js';
import fs from 'fs/promises';
import { ProxyAgent } from 'undici';
import config from '../config.js';
import { createHash } from 'crypto';


class CaptchaService {
    async new(proxy) {
        try {
            let dispatcher = new ProxyAgent(`${proxy.proto}://${proxy.username? proxy.username + "@" + proxy.password + ":" : ""}${proxy.host}:${proxy.port}`);
            
            const captcha = await (await fetch(config.captcha_source_url, {dispatcher})).blob();
            const buffer = Buffer.from(await captcha.arrayBuffer());
            const hash = createHash('md5').update(buffer).digest('hex');
            await fs.writeFile(`${config.data_dir}/${hash}.jpeg`, buffer);
            
            const id = (await db.query("INSERT INTO captchas (hash, proxy_id) VALUES ($1, $2) RETURNING id", [hash, proxy.id])).rows[0].id;

            return id
        } catch(e) {
            console.log(e);
        }
    }

    async check_solution(solution, proxy) {
        let dispatcher = new ProxyAgent(`${proxy.proto}://${proxy.username? proxy.username + "@" + proxy.password + ":" : ""}${proxy.host}:${proxy.port}`);
        const body = {
            "TotalSum": "78278",
            "FnNumber": "9960440301173139",
            "ReceiptOperationType": "1",
            "DocNumber": "35704",
            "DocFiscalSign": "4149689833",
            "Captcha": solution,
            "DocDateTime": "2022-09-21T20:28:00.000Z"
        }
        const result = await fetch("https://check.ofd.ru/Document/FetchReceiptFromFns", {
            "headers": {
                "Content-Type": "application/json;charset=utf-8"
            },
            dispatcher,
            "body": JSON.stringify(body),
            "method": "POST",
        });
        return result.status != 400;
    }

    async get (id) {
        let result = await db.query("SELECT hash FROM captchas WHERE id = $1", [id]) 
        if (result.rows[0] == undefined) {
            return undefined;
        }
        return result.rows[0].hash;
    }

    async check_and_save_solution (id, solution, proxy) {
        if (!await this.check_solution(solution, proxy)) {
            return false;
        }

        await db.query("UPDATE captchas SET solution = $1 WHERE id = $2", [solution, id]);
        return true;
    }

    async get_assigned_proxy(id) {
        return (await db.query("select proxies.id, proto, host, port, username, password from proxies JOIN captchas on proxy_id = proxies.id WHERE captchas.id = $1", [id])).rows[0];
    }
}

export default new CaptchaService();